@@ -375,7 +375,7 @@ AVAILABLE_CLI_OPTIONS = {
         help='Specify which tickers to download. Space-separated list. '
         'Default: `1m 5m`.',
         choices=['1m', '3m', '5m', '15m', '30m', '1h', '2h', '4h',
-                 '6h', '8h', '12h', '1d', '3d', '1w'],
+                 '6h', '8h', '12h', '1d', '3d', '1w', '2w', '1M', '1y'],
         default=['1m', '5m'],
         nargs='+',
     ),
@@ -455,37 +455,49 @@ AVAILABLE_CLI_OPTIONS = {
     ),
     "hyperopt_list_min_avg_time": Arg(
         '--min-avg-time',
-        help='Select epochs on above average time.',
+        help='Select epochs above average time.',
         type=float,
         metavar='FLOAT',
     ),
     "hyperopt_list_max_avg_time": Arg(
         '--max-avg-time',
-        help='Select epochs on under average time.',
+        help='Select epochs below average time.',
         type=float,
         metavar='FLOAT',
     ),
     "hyperopt_list_min_avg_profit": Arg(
         '--min-avg-profit',
-        help='Select epochs on above average profit.',
+        help='Select epochs above average profit.',
         type=float,
         metavar='FLOAT',
     ),
     "hyperopt_list_max_avg_profit": Arg(
         '--max-avg-profit',
-        help='Select epochs on below average profit.',
+        help='Select epochs below average profit.',
         type=float,
         metavar='FLOAT',
     ),
     "hyperopt_list_min_total_profit": Arg(
         '--min-total-profit',
-        help='Select epochs on above total profit.',
+        help='Select epochs above total profit.',
         type=float,
         metavar='FLOAT',
     ),
     "hyperopt_list_max_total_profit": Arg(
         '--max-total-profit',
-        help='Select epochs on below total profit.',
+        help='Select epochs below total profit.',
+        type=float,
+        metavar='FLOAT',
+    ),
+    "hyperopt_list_min_objective": Arg(
+        '--min-objective',
+        help='Select epochs above objective.',
+        type=float,
+        metavar='FLOAT',
+    ),
+    "hyperopt_list_max_objective": Arg(
+        '--max-objective',
+        help='Select epochs below objective.',
         type=float,
         metavar='FLOAT',
     ),
