@@ -3,52 +3,123 @@ Helpers when analyzing backtest data
 """
 import logging
 from pathlib import Path
-from typing import Dict, Union, Tuple
+from typing import Dict, Union, Tuple, Any, Optional
 
 import numpy as np
 import pandas as pd
 from datetime import timezone
 
 from freqtrade import persistence
+from freqtrade.constants import LAST_BT_RESULT_FN
 from freqtrade.misc import json_load
 from freqtrade.persistence import Trade
 
 logger = logging.getLogger(__name__)
 
 # must align with columns in backtest.py
-BT_DATA_COLUMNS = ["pair", "profit_percent", "open_time", "close_time", "index", "duration",
+BT_DATA_COLUMNS = ["pair", "profit_percent", "open_date", "close_date", "index", "trade_duration",
                    "open_rate", "close_rate", "open_at_end", "sell_reason"]
 
 
-def load_backtest_data(filename: Union[Path, str]) -> pd.DataFrame:
+def get_latest_backtest_filename(directory: Union[Path, str]) -> str:
     """
-    Load backtest data file.
-    :param filename: pathlib.Path object, or string pointing to the file.
-    :return: a dataframe with the analysis results
+    Get latest backtest export based on '.last_result.json'.
+    :param directory: Directory to search for last result
+    :return: string containing the filename of the latest backtest result
+    :raises: ValueError in the following cases:
+        * Directory does not exist
+        * `directory/.last_result.json` does not exist
+        * `directory/.last_result.json` has the wrong content
     """
-    if isinstance(filename, str):
-        filename = Path(filename)
+    if isinstance(directory, str):
+        directory = Path(directory)
+    if not directory.is_dir():
+        raise ValueError(f"Directory '{directory}' does not exist.")
+    filename = directory / LAST_BT_RESULT_FN
 
     if not filename.is_file():
-        raise ValueError(f"File {filename} does not exist.")
+        raise ValueError(
+            f"Directory '{directory}' does not seem to contain backtest statistics yet.")
 
     with filename.open() as file:
         data = json_load(file)
 
-    df = pd.DataFrame(data, columns=BT_DATA_COLUMNS)
+    if 'latest_backtest' not in data:
+        raise ValueError(f"Invalid '{LAST_BT_RESULT_FN}' format.")
 
-    df['open_time'] = pd.to_datetime(df['open_time'],
-                                     unit='s',
-                                     utc=True,
-                                     infer_datetime_format=True
-                                     )
-    df['close_time'] = pd.to_datetime(df['close_time'],
-                                      unit='s',
-                                      utc=True,
-                                      infer_datetime_format=True
-                                      )
-    df['profit'] = df['close_rate'] - df['open_rate']
-    df = df.sort_values("open_time").reset_index(drop=True)
+    return data['latest_backtest']
+
+
+def load_backtest_stats(filename: Union[Path, str]) -> Dict[str, Any]:
+    """
+    Load backtest statistics file.
+    :param filename: pathlib.Path object, or string pointing to the file.
+    :return: a dictionary containing the resulting file.
+    """
+    if isinstance(filename, str):
+        filename = Path(filename)
+    if filename.is_dir():
+        filename = filename / get_latest_backtest_filename(filename)
+    if not filename.is_file():
+        raise ValueError(f"File {filename} does not exist.")
+    logger.info(f"Loading backtest result from {filename}")
+    with filename.open() as file:
+        data = json_load(file)
+
+    return data
+
+
+def load_backtest_data(filename: Union[Path, str], strategy: Optional[str] = None) -> pd.DataFrame:
+    """
+    Load backtest data file.
+    :param filename: pathlib.Path object, or string pointing to a file or directory
+    :param strategy: Strategy to load - mainly relevant for multi-strategy backtests
+                     Can also serve as protection to load the correct result.
+    :return: a dataframe with the analysis results
+    :raise: ValueError if loading goes wrong.
+    """
+    data = load_backtest_stats(filename)
+    if not isinstance(data, list):
+        # new, nested format
+        if 'strategy' not in data:
+            raise ValueError("Unknown dataformat.")
+
+        if not strategy:
+            if len(data['strategy']) == 1:
+                strategy = list(data['strategy'].keys())[0]
+            else:
+                raise ValueError("Detected backtest result with more than one strategy. "
+                                 "Please specify a strategy.")
+
+        if strategy not in data['strategy']:
+            raise ValueError(f"Strategy {strategy} not available in the backtest result.")
+
+        data = data['strategy'][strategy]['trades']
+        df = pd.DataFrame(data)
+        df['open_date'] = pd.to_datetime(df['open_date'],
+                                         utc=True,
+                                         infer_datetime_format=True
+                                         )
+        df['close_date'] = pd.to_datetime(df['close_date'],
+                                          utc=True,
+                                          infer_datetime_format=True
+                                          )
+    else:
+        # old format - only with lists.
+        df = pd.DataFrame(data, columns=BT_DATA_COLUMNS)
+
+        df['open_date'] = pd.to_datetime(df['open_date'],
+                                         unit='s',
+                                         utc=True,
+                                         infer_datetime_format=True
+                                         )
+        df['close_date'] = pd.to_datetime(df['close_date'],
+                                          unit='s',
+                                          utc=True,
+                                          infer_datetime_format=True
+                                          )
+        df['profit_abs'] = df['close_rate'] - df['open_rate']
+    df = df.sort_values("open_date").reset_index(drop=True)
     return df
 
 
@@ -62,9 +133,9 @@ def analyze_trade_parallelism(results: pd.DataFrame, timeframe: str) -> pd.DataF
     """
     from freqtrade.exchange import timeframe_to_minutes
     timeframe_min = timeframe_to_minutes(timeframe)
-    dates = [pd.Series(pd.date_range(row[1].open_time, row[1].close_time,
+    dates = [pd.Series(pd.date_range(row[1]['open_date'], row[1]['close_date'],
                                      freq=f"{timeframe_min}min"))
-             for row in results[['open_time', 'close_time']].iterrows()]
+             for row in results[['open_date', 'close_date']].iterrows()]
     deltas = [len(x) for x in dates]
     dates = pd.Series(pd.concat(dates).values, name='date')
     df2 = pd.DataFrame(np.repeat(results.values, deltas, axis=0), columns=results.columns)
@@ -90,21 +161,26 @@ def evaluate_result_multi(results: pd.DataFrame, timeframe: str,
     return df_final[df_final['open_trades'] > max_open_trades]
 
 
-def load_trades_from_db(db_url: str) -> pd.DataFrame:
+def load_trades_from_db(db_url: str, strategy: Optional[str] = None) -> pd.DataFrame:
     """
     Load trades from a DB (using dburl)
     :param db_url: Sqlite url (default format sqlite:///tradesv3.dry-run.sqlite)
+    :param strategy: Strategy to load - mainly relevant for multi-strategy backtests
+                     Can also serve as protection to load the correct result.
     :return: Dataframe containing Trades
     """
-    trades: pd.DataFrame = pd.DataFrame([], columns=BT_DATA_COLUMNS)
     persistence.init(db_url, clean_open_orders=False)
 
-    columns = ["pair", "open_time", "close_time", "profit", "profit_percent",
-               "open_rate", "close_rate", "amount", "duration", "sell_reason",
+    columns = ["pair", "open_date", "close_date", "profit", "profit_percent",
+               "open_rate", "close_rate", "amount", "trade_duration", "sell_reason",
                "fee_open", "fee_close", "open_rate_requested", "close_rate_requested",
                "stake_amount", "max_rate", "min_rate", "id", "exchange",
                "stop_loss", "initial_stop_loss", "strategy", "timeframe"]
 
+    filters = []
+    if strategy:
+        filters.append(Trade.strategy == strategy)
+
     trades = pd.DataFrame([(t.pair,
                             t.open_date.replace(tzinfo=timezone.utc),
                             t.close_date.replace(tzinfo=timezone.utc) if t.close_date else None,
@@ -123,16 +199,16 @@ def load_trades_from_db(db_url: str) -> pd.DataFrame:
                             t.stop_loss, t.initial_stop_loss,
                             t.strategy, t.timeframe
                             )
-                           for t in Trade.get_trades().all()],
+                           for t in Trade.get_trades(filters).all()],
                           columns=columns)
 
     return trades
 
 
 def load_trades(source: str, db_url: str, exportfilename: Path,
-                no_trades: bool = False) -> pd.DataFrame:
+                no_trades: bool = False, strategy: Optional[str] = None) -> pd.DataFrame:
     """
-    Based on configuration option "trade_source":
+    Based on configuration option 'trade_source':
     * loads data from DB (using `db_url`)
     * loads data from backtestfile (using `exportfilename`)
     :param source: "DB" or "file" - specify source to load from
@@ -148,7 +224,7 @@ def load_trades(source: str, db_url: str, exportfilename: Path,
     if source == "DB":
         return load_trades_from_db(db_url)
     elif source == "file":
-        return load_backtest_data(exportfilename)
+        return load_backtest_data(exportfilename, strategy)
 
 
 def extract_trades_of_period(dataframe: pd.DataFrame, trades: pd.DataFrame,
@@ -163,11 +239,31 @@ def extract_trades_of_period(dataframe: pd.DataFrame, trades: pd.DataFrame,
     else:
         trades_start = dataframe.iloc[0]['date']
         trades_stop = dataframe.iloc[-1]['date']
-    trades = trades.loc[(trades['open_time'] >= trades_start) &
-                        (trades['close_time'] <= trades_stop)]
+    trades = trades.loc[(trades['open_date'] >= trades_start) &
+                        (trades['close_date'] <= trades_stop)]
     return trades
 
 
+def calculate_market_change(data: Dict[str, pd.DataFrame], column: str = "close") -> float:
+    """
+    Calculate market change based on "column".
+    Calculation is done by taking the first non-null and the last non-null element of each column
+    and calculating the pctchange as "(last - first) / first".
+    Then the results per pair are combined as mean.
+
+    :param data: Dict of Dataframes, dict key should be pair.
+    :param column: Column in the original dataframes to use
+    :return:
+    """
+    tmp_means = []
+    for pair, df in data.items():
+        start = df[column].dropna().iloc[0]
+        end = df[column].dropna().iloc[-1]
+        tmp_means.append((end - start) / start)
+
+    return np.mean(tmp_means)
+
+
 def combine_dataframes_with_mean(data: Dict[str, pd.DataFrame],
                                  column: str = "close") -> pd.DataFrame:
     """
@@ -190,7 +286,7 @@ def create_cum_profit(df: pd.DataFrame, trades: pd.DataFrame, col_name: str,
     """
     Adds a column `col_name` with the cumulative profit for the given trades array.
     :param df: DataFrame with date index
-    :param trades: DataFrame containing trades (requires columns close_time and profit_percent)
+    :param trades: DataFrame containing trades (requires columns close_date and profit_percent)
     :param col_name: Column name that will be assigned the results
     :param timeframe: Timeframe used during the operations
     :return: Returns df with one additional column, col_name, containing the cumulative profit.
@@ -201,7 +297,7 @@ def create_cum_profit(df: pd.DataFrame, trades: pd.DataFrame, col_name: str,
     from freqtrade.exchange import timeframe_to_minutes
     timeframe_minutes = timeframe_to_minutes(timeframe)
     # Resample to timeframe to make sure trades match candles
-    _trades_sum = trades.resample(f'{timeframe_minutes}min', on='close_time'
+    _trades_sum = trades.resample(f'{timeframe_minutes}min', on='close_date'
                                   )[['profit_percent']].sum()
     df.loc[:, col_name] = _trades_sum.cumsum()
     # Set first value to 0
@@ -211,13 +307,13 @@ def create_cum_profit(df: pd.DataFrame, trades: pd.DataFrame, col_name: str,
     return df
 
 
-def calculate_max_drawdown(trades: pd.DataFrame, *, date_col: str = 'close_time',
+def calculate_max_drawdown(trades: pd.DataFrame, *, date_col: str = 'close_date',
                            value_col: str = 'profit_percent'
                            ) -> Tuple[float, pd.Timestamp, pd.Timestamp]:
     """
     Calculate max drawdown and the corresponding close dates
-    :param trades: DataFrame containing trades (requires columns close_time and profit_percent)
-    :param date_col: Column in DataFrame to use for dates (defaults to 'close_time')
+    :param trades: DataFrame containing trades (requires columns close_date and profit_percent)
+    :param date_col: Column in DataFrame to use for dates (defaults to 'close_date')
     :param value_col: Column in DataFrame to use for values (defaults to 'profit_percent')
     :return: Tuple (float, highdate, lowdate) with absolute max drawdown, high and low time
     :raise: ValueError if trade-dataframe was found empty.
